/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.reqcoverage.eclipse.wizards;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.unitesk.requality.eclipse.wizards.ITestGenWizard;

public class NewTestPage extends WizardPage {
  private Text containerText, containerViewText;
  private ModifyListener ml;
  /**
   * Constructor for SampleNewWizardPage.
   *
   * @param pageName
   */
  public ITestGenWizard getTestWizard() {
    return (ITestGenWizard) getWizard();
  }

  public NewTestPage(ISelection selection) {
    super("newTestTextPage");
    setPageComplete(false);
    setDescription("Please verify test file contents.");
  }

  /** @see IDialogPage#createControl(Composite) */
  private String textback;

  public void createControl(Composite parent) {
    Composite container = new Composite(parent, SWT.NULL);
    GridLayout layout = new GridLayout();
    container.setLayout(layout);
    layout.numColumns = 1;
    layout.verticalSpacing = 9;
    Label label = new Label(container, SWT.NULL);
    label.setText("&Text:");

    containerText = new Text(container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
    String qid = storedtargetQiD;
    if (qid == null) qid = ((ITestGenWizard) getWizard()).getTargetQID();
    if (qid == null) qid = getDefaultQID();
    GridData gd = new GridData(GridData.FILL_BOTH);
    containerText.setLayoutData(gd);
    ml =
        new ModifyListener() {
          public void modifyText(ModifyEvent e) {
            dialogChanged();
          }
        };
    containerText.addModifyListener(ml);
    containerText.setFocus();

    label = new Label(container, SWT.NULL);
    label.setText("&Preview:");

    containerViewText = new Text(container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
    containerViewText.setEditable(false);
    gd = new GridData(GridData.FILL_BOTH);
    containerViewText.setLayoutData(gd);

    if (textback != null) containerText.setText(textback);
    setTargetQID(qid, true);

    setControl(container);
    dialogChanged();
  }

  private String getDefaultQID() {
    return "Element Qualifying Id";
  }

  /** Ensures that both text fields are set. */
  private void dialogChanged() {
    //TODO check
    updateStatus(null);
    String qid = ((ITestGenWizard) getWizard()).getTargetQID();
    if (qid == null) qid = getDefaultQID();
    if (containerViewText != null)
      containerViewText.setText(containerText.getText().replace("${nodeQID}", qid));
  }

  private void updateStatus(String message) {
    setErrorMessage(message);
    setPageComplete(message == null);
  }

  public String getRawContents() {
    return containerText.getText();
  }

  public String getFileContents() {
    return containerViewText.getText();
  }

  private String storedtargetQiD;

  public void setTargetQID(String targetqid, boolean local) {
    if (targetqid == null) targetqid = getDefaultQID();
    if (targetqid == null && storedtargetQiD != null || 
        targetqid != null && !(targetqid.equals(storedtargetQiD))) {
      storedtargetQiD = targetqid;
      if (containerText != null && containerViewText != null)
        containerViewText.setText(containerText.getText().replace("${nodeQID}", targetqid));
    }
  }

  public void updateSettings() {
    setTitle(getTestWizard().getSettings().get("title"));
    if (textback == null) textback = getTestWizard().getSettings().get("template");
    if (containerText != null) containerText.setText(textback);
  }

  public void setTSDescription(String savedText) {
    textback = savedText;
  }
}
