/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.reqcoverage.reqstracer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.Path;
import com.unitesk.requality.core.RLogger;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class Collector {

  List<File> testProjectsList = new ArrayList<File>();
  ArrayList<String> extensions = new ArrayList<String>();
  ArrayList<String> regExpressions = new ArrayList<String>();
  IProject requalityProject;
  ArrayList<File> filesToFindReqs = new ArrayList<File>();
  HashMap<String, ArrayList<String>> mapReqsToFiles = new HashMap<String, ArrayList<String>>();
  HashMap<String, ArrayList<String>> mapNameToIds = new HashMap<String, ArrayList<String>>();

  private static Collector _instance = new Collector();

  public static Collector instance() {
    return _instance;
  }

  public ArrayList<File> getFilesToFindReqs() {
    return filesToFindReqs;
  }
  //
  //	public void setFilesToFindReqs(ArrayList<IFile> filesToFindReqs) {
  //		this.filesToFindReqs = filesToFindReqs;
  //	}

  public IProject getRequalityProject() {
    return requalityProject;
  }

  public void setRequalityProject(IProject requalityProject) {
    this.requalityProject = requalityProject;
  }

  public List<File> getTestProjectsList() {
    return testProjectsList;
  }

  public void setContainersList(List<File> testProjectsList) {
    this.testProjectsList = testProjectsList;
  }

  public ArrayList<String> getExtensions() {
    return extensions;
  }

  public void setExtensions(ArrayList<String> extensions) {
    this.extensions = extensions;
  }

  public ArrayList<String> getRegExpressions() {
    return regExpressions;
  }

  public void setRegExpressions(ArrayList<String> regExpressions) {
    this.regExpressions = regExpressions;
  }

  public String listFiles(File iproj, ArrayList<File> tempResult) throws CoreException {
    String ret = "";
    if (!iproj.exists()) return iproj.getName() + " is not exists, ";
    //if(!iproj.getProject().isAccessible())
    //return iproj.getName() +" is not accessible, ";
    for (File res : iproj.listFiles()) { //(.members()) {
      if (res.isDirectory()) {
        ret += listFiles(res, tempResult);
      } else {
        tempResult.add(res);
      }
    }
    return ret;
  }

  public boolean isInExtList(String extension) {
    for (String ext : extensions) {
      if (ext.equalsIgnoreCase(extension)) {
        return true;
      }
    }
    return false;
  }

  public String selectFilesToFindReqs() throws CoreException {
    ArrayList<File> tempResult = new ArrayList<File>();
    boolean exists = false;
    String ret = "";
    for (File iproj : testProjectsList) {
      //REFRESH problem
      //iproj.refreshLocal(IProject.DEPTH_INFINITE, null);
      ret += listFiles(iproj, tempResult);
    }
    //TODO remove
    //		RLogger.logInfo("tempResult = " + tempResult);

    for (File ifile : tempResult) {
      String ext = "";
      if (ifile.getName().lastIndexOf(".") > -1)
        ext = ifile.getName().substring(ifile.getName().lastIndexOf(".") + 1);
      if (isInExtList(ext)) {
        filesToFindReqs.add(ifile);
      }
    }
    return ret;
    //TODO remove
    //		RLogger.logInfo("filesToFindReqs = " + filesToFindReqs);
  }

  public void constructMapReqsToFiles() throws CoreException, IOException {
    for (File ifile : filesToFindReqs) {
      BufferedReader in = new BufferedReader(new FileReader(ifile.getAbsoluteFile()));
      String strLine;
      ;
      Pattern[] patterns = new Pattern[regExpressions.size()];
      int count = 0;
      for (String str : regExpressions) {
        patterns[count++] = Pattern.compile(str);
      }
      while ((strLine = in.readLine()) != null) {
        //				  RLogger.logInfo (strLine);
        for (Pattern pattern : patterns) {
          Matcher matcher = pattern.matcher(strLine);
          if (matcher.find()) {
            //TODO �������� ������ �����
            for (int i = 1; i <= matcher.groupCount(); i++) {
              String req = matcher.group(i);
              //						  RLogger.logInfo("mathes req = " + req);
              if (mapReqsToFiles.containsKey(req)) {
                ArrayList<String> files = mapReqsToFiles.get(req);
                files.add(ifile.getAbsolutePath());
                //							  RLogger.logInfo("added file " + ifile.getLocation().toOSString() + " to map for req = " + req);
              } else {
                ArrayList<String> files = new ArrayList<String>();
                files.add(ifile.getAbsolutePath());
                mapReqsToFiles.put(req, files);
              }
            }
          }
        }
      }
      in.close();
    }
    //		RLogger.logInfo("mapReqsToFiles = " + mapReqsToFiles);
  }

  public ArrayList<TreeNode> getFirstLevelNodes() {
    DeepFirstTreeWalker nodes = new DeepFirstTreeWalker(getTreeDB().getRootNode());
    ArrayList<TreeNode> result = new ArrayList<TreeNode>();
    for (TreeNode node : nodes) {
      if ((node.getType().equals("Requirement"))) {
        if (!((((Requirement) node)).getQualifiedId()).equals("/Requirements")) {
          if (node.getShownParent().equals(getTreeDB().getNode("/Requirements", Path.QID))) {
            result.add(node);
          }
        }
      }
    }
    return result;
  }

  public void makeNameIdMap() {
    for (TreeNode n : new DeepFirstTreeWalker(getTreeDB().getRootNode())) {
      if (n.getType().equals("Requirement")) {
        Requirement req = (Requirement) n;
        //	                String name = req.getName();
        String name = getFullNodeName(n);
        namesToIds.put(name, n.getId());
        if (mapNameToIds.get(name) != null) {
          ArrayList<String> ids = mapNameToIds.get(name);
          ids.add(n.getQualifiedId());
          mapNameToIds.put(name, ids);
        } else {
          ArrayList<String> ids = new ArrayList<String>();
          ids.add(n.getQualifiedId());
          mapNameToIds.put(name, ids);
        }
      } else if (n.getType().equals("TestPurpose")) {
        // String qId = n.getQualifiedId();
        // String name = qId.substring(14);
        String name = getFullNodeName(n);

        if (mapNameToIds.get(name) != null) {
          ArrayList<String> ids = mapNameToIds.get(name);
          ids.add(n.getQualifiedId());
          mapNameToIds.put(name, ids);
        } else {
          ArrayList<String> ids = new ArrayList<String>();
          ids.add(n.getQualifiedId());
          mapNameToIds.put(name, ids);
        }
      }
    }
  }

  public String getValue(String key) {
    ArrayList<String> resultSet = mapReqsToFiles.get(key);
    if (resultSet != null) {
      StringBuilder result = new StringBuilder();
      for (String str : resultSet) {
        result.append(str);
        result.append("\r\n");
      }
      return result.toString();
    }
    return "";
  }

  public HashMap<String, ArrayList<String>> getReqsToFilesMap() {
    return mapReqsToFiles;
  }

  public void calculateInconclusive() {
    label:
    for (String req : mapReqsToFiles.keySet()) {
      if (req.length() < 14) {
        inconclusive.add(req);
        continue label;
      }
      String req1 = req.substring(14);
      if (req1.contains("/")) {

        String reqId = namesToIds.get(req1.subSequence(0, req1.indexOf("/")));

        if (reqId == null) {
          inconclusive.add(req);
          continue label;
        }
        String name =
            namesToIds.get(req1.subSequence(0, req1.indexOf("/")))
                + req1.substring(req1.indexOf("/"));
        if (!mapNameToIds.containsKey(name)) {
          inconclusive.add(req);
          continue label;
        }
      } else {
        inconclusive.add(req);
      }
    }
    //		 RLogger.logInfo("inconclusive = " + inconclusive);
  }

  public void reset() {
    reqsToMethods.clear();
    DBName = null;
    pathToReqDB = null;
    // treeDB = null;
    setTreeDB(null);
    mapNameToIds.clear();
    //		useDB = false;
    purposesCount = 0;
    reqsCount = 0;
    reqsNames.clear();
    mapReqNamesToCellClass.clear();
    reqsDBCount = 0;
    tpDBCount = 0;
    processReport = false;
    //		countInconclusive = 0;
    inconclusive.clear();
    namesToIds.clear();

    testProjectsList.clear();
    extensions.clear();
    regExpressions.clear();
    requalityProject = null;
    filesToFindReqs.clear();
    mapReqsToFiles.clear();
  }

  //----------------------------------------------------

  public HashMap<String, HashSet<String>> reqsToMethods = new HashMap<String, HashSet<String>>();
  public String DBName;
  public String pathToReqDB;
  public TreeDB _treeDB;

  //	public boolean useDB;

  public HashMap<String, String> namesToIds =
      new HashMap<String, String>(); // ����������� �� ���� ���������� � id

  public int purposesCount;
  public int reqsCount;
  private int reqsDBCount;
  private int tpDBCount;
  //	private int countInconclusive;

  public ArrayList<String> inconclusive = new ArrayList<String>();
  public Set<String> reqsNames = new HashSet<String>();
  public HashMap<String, String> mapReqNamesToCellClass = new HashMap<String, String>(); //
  // // ���������� "covered", "uncovered1", "partcovered" ��� "nondeterministic"
  public boolean processReport;

  public boolean isProcessReport() {
    return processReport;
  }

  public void setProcessReport(boolean processReport) {
    this.processReport = processReport;
  }

  public String getPathToReqDB() {
    return pathToReqDB;
  }

  //	public boolean isUseDB() {
  //		return useDB;
  //	}
  //
  //	public void setUseDB(boolean useDB) {
  //		this.useDB = useDB;
  //	}

  public void setTreeDB(TreeDB db) {
    _treeDB = db;
  }

  public TreeDB getTreeDB() {
    return _treeDB;
  }

  public HashMap<String, HashSet<String>> getReqsToMethods() {
    return reqsToMethods;
  }

  public void setReqsToMethods(HashMap<String, HashSet<String>> reqsToMethods) {
    this.reqsToMethods = reqsToMethods;
  }

  public Set<String> getKeys() {
    TreeMap<String, HashSet<String>> result =
        new TreeMap<String, HashSet<String>>(String.CASE_INSENSITIVE_ORDER);
    result.putAll(reqsToMethods);
    return result.keySet();
  }

  public String getCellClass(String key) {
    //		String result = mapReqNamesToCellClass.get(key);
    String result = mapReqNamesToCellClass.get("/Requirements/" + key);
    if (result != null) return result;
    return "";
  }

  public String getDBName() {
    if (DBName == null) return "";
    return DBName;
  }

  public void setDBName(String name) {
    DBName = name;
  }

  public int getAllChildrenSize(TreeNode node) {
    int result = 0;
    for (int i = 0; i < node.sizeShownChildren(); i++)
      result = result + 1 + getAllChildrenSize(node.getShownChild(i));

    return result;
  }

  public int getRowspan(TreeNode node) {
    return (getAllChildrenSize(node) + 1);
  }

  public String getNameForPrint(String qId) {
    String[] reqIds = qId.substring(14).split("/");
    String name = "";
    String tempId = "/Requirements";
    for (int i = 0; i < reqIds.length; i++) {
      tempId = tempId + "/" + reqIds[i];
      TreeNode tn = getTreeDB().getNode(tempId, Path.QID | Path.UV);
      if (tn.getType().equals("Requirement")) {
        String partReqName = ((Requirement) tn).getName();
        name = name + "/" + partReqName;
      } else name = name + "/" + reqIds[reqIds.length - 1];
    }
    return name.substring(1);
  }

  public String getDescription(TreeNode node) {
    String result = "";
    if (node.getType().equals("Requirement")) {
      Requirement req = (Requirement) node;
      if (req.getDescription() != null && !req.getDescription().equals("")) {
        result = req.getDescription();
      } else {
        for (Location loc : req.getLocations()) {
          result = result + " " + loc.getDescription();
        }
      }
    }

    if (node.getType().equals("TestPurpose")) {
      result = node.getDescription();
    }

    return result;
  }

  public boolean hasChildren(TreeNode node) {
    if (node.sizeShownChildren() == 0) {
      return false;
    } else {
      return true;
    }
  }

  public ArrayList<TreeNode> getAllChildren(TreeNode parent) {
    if (!hasChildren(parent)) return null;

    ArrayList<TreeNode> result = new ArrayList<TreeNode>();
    for (int i = 0; i < parent.sizeShownChildren(); i++) {
      result.add(parent.getShownChild(i));
      if (parent.getShownChild(i).sizeShownChildren() != 0) {
        result.addAll(getAllChildren(parent.getShownChild(i)));
      }
    }
    return result;
  }

  public String getShortNameForPrint(String qId) {
    TreeNode node = getTreeDB().getNode(qId, Path.QID | Path.UV);
    if (node.getType().equals("TestPurpose")) {
      return qId.substring(qId.lastIndexOf("/") + 1);
    }

    String[] reqIds = qId.substring(14).split("/");
    String name = "";
    String tempId = "/Requirements";
    for (int i = 0; i < reqIds.length; i++) {
      tempId = tempId + "/" + reqIds[i];

      TreeNode tn = getTreeDB().getNode(tempId, Path.QID | Path.UV);
      if (tn.getType().equals("Requirement")) {
        String partReqName = ((Requirement) node).getName();
        name = partReqName.substring(partReqName.lastIndexOf("/") + 1);
      }
    }
    return name;
  }

  public String getFullTPName(String reqName, String tpName) {
    String result = reqName + "/" + tpName;
    return result;
  }

  //    public void calculateCoverage() {
  //    	Set<String> testPurposesSet = reqsToMethods.keySet();
  //    	calculateCellClasses(); // ������ ����������� �� ���� ���������� � ����� ������ ��� html
  //    	calculateReqs(); // ������� ���������� test purposes, req � �������������� ���������� � ������������� .py �������
  //
  //    	for (String str: testPurposesSet) {
  //    		if (str.contains("/")) {
  //    			purposesCount++;
  //    		} else {
  //    			reqsNames.add(str); // ������ ���� ����������, ������� ����� ��������� �� TestPurposes, ������� �� ���
  //    		}
  //    	}
  //
  //    	label2:
  //    	for (String reqName: reqsNames) {
  //    		ArrayList<String> reqIds = mapNameToIds.get(reqName);
  //    		if (reqIds == null) {
  //    			inconclusive.add(reqName);
  //				continue label2;
  //    		}
  //    		if (reqIds.size() == 1) {
  //    			String qId = reqIds.get(0);
  //    			TreeNode node = getTreeDb().getNode(qId);
  //    			// TODO �������� �� ����������� ������ ���� (ex, Req/Req/TP)
  //    			if (!node.getType().equalsIgnoreCase("Requirement")) {
  //    				continue;
  //    			}
  //    			Requirement req = (Requirement) node;
  //    			mapReqNamesToCellClass.put(reqName, "covered");
  //    		} else  { 	// ����� ������ ���������� � ����� ������ - ������, ������� nondeterministic
  //    			mapReqNamesToCellClass.put(reqName, "nondeterministic");
  //    		}
  //    	}
  //
  //    	Set<String> reqsAndTPs = mapNameToIds.keySet();
  //
  //    	for (String name: reqsAndTPs) {
  //    		ArrayList<String> ids = mapNameToIds.get(name);
  //    		String qId = ids.get(0);
  //			TreeNode node = getTreeDb().getNode(qId);
  //    		if (ids.size() == 1) {
  //    			if (node.getType().equalsIgnoreCase("Requirement")) {
  //        			if (getReqCellClass((Requirement)node).equals("covered")) {
  //    				reqsCount++;
  //        			}
  //    			}
  //    		}
  //    	}
  //    }

  public void calculateCellClasses() {
    // ����������� �� ���� ����� �������� ���������� � ������� ��������� � ������������ ������� �������� �������
    Set<String> reqsAndTPs = mapNameToIds.keySet();
    for (String name : reqsAndTPs) {
      ArrayList<String> ids = mapNameToIds.get(name);
      if (ids.size() == 1) {
        String qId = ids.get(0);
        TreeNode node = getTreeDB().getNode(qId, Path.QID | Path.UV);
        if (node.getType().equalsIgnoreCase("TestPurpose")) {
          //TODO remove
          //String reqNameShort = getTPName(node);
          // String reqName = "/Requirements/" + reqNameShort;
          //    				String reqName = getFullNodeName(node);
          // � ����������� mapNameToIds ��� �������� ������ �����
          String reqName = name;
          //    				RLogger.logInfo("calculateCellClasses: reqName = " + reqName);
          //    				if (reqsToMethods.containsKey(reqName)) {
          RLogger.logInfo("Try requirement: " + reqName);
          if (mapReqsToFiles.containsKey(reqName)) {
            //        				RLogger.logInfo(": covered!" );
            mapReqNamesToCellClass.put(reqName, "covered");
          } else {
            //        				RLogger.logInfo(": uncovered:(" );
            mapReqNamesToCellClass.put(reqName, "uncovered1");
          }
        }
        if (node.getType().equalsIgnoreCase("Requirement")) {
          Requirement req = (Requirement) node;
          mapReqNamesToCellClass.put(name, getReqCellClass(req));
        }
      } else { // ����� ������ ���������� � ����� ������ - ������, ������� nondeterministic
        mapReqNamesToCellClass.put(name, "nondeterministic");
      }
    }
    //    	RLogger.logInfo("mapReqNamesToCellClass = " + mapReqNamesToCellClass);
  }

  private String getNodeName(TreeNode node) {
    if (node.getType().equalsIgnoreCase("Requirement")) {
      return ((Requirement) node).getName();
    }
    if (node.getType().equalsIgnoreCase("TestPurpose")) {
      return node.getId();
    } else {
      throw new IllegalArgumentException("Unexpected node type: " + node.getType());
    }
  }

  private String getFullNodeName(TreeNode node) {
    StringBuilder result = new StringBuilder();
    while (node != null && !"DBRoot".equals(node.getType())) {
      result.insert(0, getNodeName(node));
      result.insert(0, '/');
      node = node.getShownParent();
    }
    return result.toString();
  }

  private String getTPName(TreeNode node) {
    if (node.getType().equalsIgnoreCase("Requirement")) return ((Requirement) node).getName();
    //    		return "/Requirements/" + ((Requirement)node).getName();
    if (node.getType().equalsIgnoreCase("TestPurpose")) {
      return getTPName(node.getShownParent()) + "/" + node.getId();
      //    		return "/Requirements/" + getTPName(node.getParent()) + "/" + node.getId();
    }
    return "";
    //    	return "/Requirements/";
  }

  public String getRequirementsAndTPName(TreeNode node) {
    //TODO remove
    //    	RLogger.logInfo("getRequirementsAndTPName, node = " + node.getQualifiedId());
    if (node.getType().equalsIgnoreCase("Requirement"))
      return "/Requirements/" + ((Requirement) node).getName();
    if (node.getType().equalsIgnoreCase("TestPurpose")) {
      return "/Requirements/" + getTPName(node.getShownParent()) + "/" + node.getId();
    }
    return "";
  }

  public String getReqCellClass(
      Requirement req) { // ���������� "covered", "uncovered1", "partcovered"
    int childCovered = 0;
    ArrayList<TreeNode> children = getAllChildren(req);
    if (children != null) {
      for (TreeNode node : children) {
        if (node.getType().equalsIgnoreCase("Requirement")) { // ���� ������� ����� ����������
          String childRes = getReqCellClass((Requirement) node);
          if (childRes.equals("covered")) {
            childCovered++;
            continue;
          }
        } else { // ����� ������� - TestPurpose
          HashSet<String> methods = reqsToMethods.get(getNameForPrint(node.getQualifiedId()));
          if (methods != null) {
            childCovered++;
          }
        }
      }
    }
    if (childCovered == getAllChildrenSize(req)) {
      return "covered";
    }
    if (childCovered == 0) {
      return "uncovered1";
    }
    return "partcovered";
  }

  public String getGlobClass(TreeNode node) {
    int covered = 0;
    if (getAllChildren(node) != null) {
      for (TreeNode tn : getAllChildren(node)) {
        String nodeName =
            getFullTPName(
                getNameForPrint(node.getQualifiedId()), getShortNameForPrint(tn.getQualifiedId()));
        String cellClass = getCellClass(nodeName);
        if (cellClass.equals("covered")) covered++;
      }
    }
    String nodeName = getNameForPrint(node.getQualifiedId());
    String status = mapReqNamesToCellClass.get(nodeName);
    if (status == null) {
      if (covered == getAllChildrenSize(node)) return "covered";
    } else if (status.equalsIgnoreCase("covered")) {
      if (covered == getAllChildrenSize(node)) return "covered";
      else return "nondeterministic";
    } else if (status.equalsIgnoreCase("partcovered")) return "partcovered";

    return "uncovered1";
  }

  public void calculateReqs() {
    DeepFirstTreeWalker nodes = new DeepFirstTreeWalker(getTreeDB().getRootNode());
    for (TreeNode node : nodes) {
      if ((node.getType().equals("Requirement"))) {
        reqsDBCount++;
      }
      if ((node.getType().equals("TestPurpose"))) {
        tpDBCount++;
      }
    }

    label:
    for (String req : reqsToMethods.keySet()) {
      if (req.contains("/")) {
        String reqId = namesToIds.get(req.subSequence(0, req.indexOf("/")));
        if (reqId == null) {
          inconclusive.add(req);
          continue label;
        }
        String name =
            namesToIds.get(req.subSequence(0, req.indexOf("/"))) + req.substring(req.indexOf("/"));
        if (!mapNameToIds.containsKey(name)) {
          inconclusive.add(req);
          continue label;
        }
      } //--------- ���� ���������� �������� ������ (�� �������� /) , �� ��� ����������� �����, � ������ calculateCoverage
    }
  }

  public String getReqCoverage() {
    if (reqsDBCount != 0) return "" + 100 * reqsCount / reqsDBCount;
    return "unknown";
  }

  public String getTPCoverage() {

    if (tpDBCount != 0) return "" + 100 * (purposesCount - inconclusive.size()) / tpDBCount;
    return "unknown";
  }

  public String getCoveredTP() {
    return "" + (purposesCount - inconclusive.size());
  }

  public String getTotalTP() {
    return "" + tpDBCount;
  }

  public String getCoveredReqs() {
    return "" + reqsCount;
  }

  public String getTotalReqs() {
    return "" + reqsDBCount;
  }

  public int getInconclusiveCount() {
    return inconclusive.size();
  }

  public ArrayList<String> getInconclusive() {
    ArrayList<String> result = inconclusive;
    Collections.sort(result);
    return result;
  }

  public String getClassCovered(String str) {
    //		RLogger.logInfo("getClassCovered str = '" + str + "'");
    if (str != "") return "covered";
    return "uncovered1";
  }
}
