package com.unitesk.reqcoverage.reqstracer;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.model.WorkbenchLabelProvider;

public class SetupPage extends WizardPage {
  private Text extensions;
  private Composite container;
  private Text regEx;

  public SetupPage() {
    super("Report setup page");
    setTitle("Setup report");
    setDescription("Please specify parameters of the requirements coverage report");
  }

  public Set<String> getExtensions() {
    String str = extensions.getText();
    String[] extensions = str.split(",");
    HashSet<String> result = new HashSet<String>();
    for (String estr : extensions) {
      result.add(estr.trim());
    }
    return result;
  }

  public String getRegEx() {
    return regEx.getText();
  }

  @Override
  public void createControl(Composite parent) {
    container = new Composite(parent, SWT.NULL);
    GridLayout layout = new GridLayout();
    container.setLayout(layout);
    layout.numColumns = 1;

    createProjectSelector();
    addExtensionControls();
    addRegexControls();

    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    extensions.setLayoutData(gd);
    // Required to avoid an error in the system
    setControl(container);
  }

  private void addRegexControls() {
    Composite frame = new Composite(container, SWT.BORDER);
    GridLayout layout = new GridLayout(2, false);
    frame.setLayout(layout);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = getNumColumns();
    frame.setLayoutData(gd);

    Label label1 = new Label(frame, SWT.NULL);
    label1.setText("Regular expression");

    regEx = new Text(frame, SWT.BORDER);
    regEx.setText("(/Requirements/.*)");

    GridData gd2 = new GridData(GridData.FILL_HORIZONTAL);
    regEx.setLayoutData(gd2);
  }

  private void addExtensionControls() {
    Composite frame = new Composite(container, SWT.BORDER);
    GridLayout layout = new GridLayout(2, false);
    frame.setLayout(layout);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = getNumColumns();
    frame.setLayoutData(gd);

    Label label1 = new Label(frame, SWT.NULL);
    label1.setText("Tests extensions");

    extensions = new Text(frame, SWT.BORDER);
    extensions.setText("");
    extensions.addKeyListener(
        new KeyListener() {

          @Override
          public void keyPressed(KeyEvent e) {}

          @Override
          public void keyReleased(KeyEvent e) {
            if (!extensions.getText().isEmpty()) {
              setPageComplete(true);
            }
          }
        });
  }

  private CheckboxTreeViewer projectList;

  private final class ProjectListTreeProvider implements ITreeContentProvider {
    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

    @Override
    public void dispose() {}

    @Override
    public boolean hasChildren(Object element) {
      if (element == ResourcesPlugin.getWorkspace().getRoot()) {
        return true;
      }
      return false;
    }

    @Override
    public Object getParent(Object element) {
      if (element == ResourcesPlugin.getWorkspace().getRoot()) {
        return null;
      }
      return ResourcesPlugin.getWorkspace().getRoot();
    }

    @Override
    public Object[] getElements(Object inputElement) {
      return ResourcesPlugin.getWorkspace().getRoot().getProjects();
    }

    @Override
    public Object[] getChildren(Object parentElement) {
      if (parentElement == ResourcesPlugin.getWorkspace().getRoot()) {
        return getElements(parentElement);
      }
      return new Object[0];
    }
  }

  public void createProjectSelector() {
    projectList = new CheckboxTreeViewer(container);
    GridData layoutData = new GridData(GridData.FILL_BOTH);
    layoutData.horizontalSpan = getNumColumns();
    projectList.getControl().setLayoutData(layoutData);
    projectList.setContentProvider(new ProjectListTreeProvider());
    projectList.setLabelProvider(new WorkbenchLabelProvider());
    projectList.setInput(ResourcesPlugin.getWorkspace().getRoot());
  }

  private int getNumColumns() {
    return ((GridLayout) container.getLayout()).numColumns;
  }

  public ArrayList<IProject> getSelectedProjects() {
    ArrayList<IProject> result = new ArrayList<IProject>();
    for (Object p : projectList.getCheckedElements()) {
      result.add((IProject) p);
    }
    return result;
  }
}
