package com.unitesk.reqcoverage.reqstracer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.wizard.Wizard;

public class SetupWizard extends Wizard {

  private SetupPage page;
  private boolean cancelled;

  private Pattern regExp;
  private Set<String> extensions;
  private List<IProject> selectedProjects;

  public SetupWizard() {
    super();
    setNeedsProgressMonitor(true);
    setHelpAvailable(false);
  }

  public Pattern getRegExp() {
    if (cancelled) {
      throw new IllegalStateException("The wizard was cancelled");
    }
    return regExp;
  }

  public Set<String> getExtensions() {
    if (cancelled) {
      throw new IllegalStateException("The wizard was cancelled");
    }
    return extensions;
  }

  public List<IProject> getSelectedProjects() {
    if (cancelled) {
      throw new IllegalStateException("The wizard was cancelled");
    }
    return selectedProjects;
  }

  @Override
  public void addPages() {
    page = new SetupPage();
    addPage(page);
  }

  public boolean isCancelled() {
    return cancelled;
  }

  void listFiles(IContainer i, ArrayList<IFile> result) throws CoreException {
    for (IResource res : i.members()) {
      if (res instanceof IContainer) {
        listFiles((IContainer) res, result);
      } else {
        result.add((IFile) res);
      }
    }
  }

  @Override
  public boolean performFinish() {
    //		RLogger.logInfo("in performFinish getSelectedProjects = " + page.getSelectedProjects());
    cancelled = false;
    regExp = Pattern.compile(page.getRegEx());
    extensions = page.getExtensions();
    selectedProjects = page.getSelectedProjects();
    return true;
  }

  @Override
  public boolean performCancel() {
    //		RLogger.logInfo("in performCancel getSelectedProjects = " + page.getSelectedProjects());
    cancelled = true;
    return true;
  }
}
